using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using System.Linq;
using System;

using BigSock.Service;

namespace BigSock {
    
    public partial class BringerOfDeathController : EnemyController
    {

        protected static readonly TimeSpan SPELL_CAST_COOLDOWN = new TimeSpan(0, 0, 0, 4, 0);
        protected static readonly TimeSpan CAST_START = new TimeSpan(0, 0, 0, 0, 500);
        protected static readonly TimeSpan CAST_LOCATE = new TimeSpan(0, 0, 0, 0, 150);
        protected static readonly TimeSpan CAST_FINISH = new TimeSpan(0, 0, 0, 0, 750);

        protected Vector3 spellLocation;

        protected DateTime casting_time = DateTime.Now;
        protected DateTime spell_cooldown = DateTime.Now + SPELL_CAST_COOLDOWN;

        protected static readonly TimeSpan TELEPORT_COOLDOWN = new TimeSpan(0, 0, 0, 6, 0);
        protected static readonly TimeSpan TELEPORT_START = new TimeSpan(0, 0, 0, 0, 500);
        protected static readonly TimeSpan TELEPORT_LOCATE = new TimeSpan(0, 0, 0, 0, 50);
        protected static readonly TimeSpan TELEPORT_FINISH = new TimeSpan(0, 0, 0, 0, 850);

        protected Vector3 teleportLocation;

        protected DateTime teleport_time = DateTime.Now;
        protected DateTime teleport_cooldown = DateTime.Now + TELEPORT_COOLDOWN;

        public BoDState State { get; protected set; } = BoDState.Idle;

        protected EmptyCollider spellCollider;

        protected IAttack _testAttack;
        public const string PROJECTILE_NAME = "Bosses/Attacks/Bringer_of_Death_Spell";


        protected override void Update() {
            Regenerate();

            if (target != null && DateTime.Now > spell_cooldown && (this.baseHP >= baseMaxHP*0.65 || this.baseHP < baseMaxHP*0.3)){
                casting_time = DateTime.Now + CAST_START;
                State = BoDState.CastLocate;
                m_Animator.SetTrigger("cast");
                spell_cooldown = DateTime.Now + SPELL_CAST_COOLDOWN;
            }
            else if (target != null && State == BoDState.CastLocate && DateTime.Now > casting_time) {
                casting_time = DateTime.Now + CAST_LOCATE;
                State = BoDState.Casting;
                spellLocation = target.transform.position;
            } 
            else if (State == BoDState.Casting && DateTime.Now > casting_time) {
                casting_time = DateTime.Now + CAST_FINISH;
                State = BoDState.CastFinish;
                CastSpell(spellLocation);
            } 
            else if (State == BoDState.CastFinish && DateTime.Now > casting_time) {
                m_Animator.SetTrigger("walk");
                State = BoDState.Walk;
                
            }
            else if (target != null && DateTime.Now > teleport_cooldown && this.baseHP <= baseMaxHP*0.65) {
                teleport_time = DateTime.Now + TELEPORT_START;
                State = BoDState.TeleportLocate;
                m_Animator.SetTrigger("death");
                teleport_cooldown = DateTime.Now + TELEPORT_COOLDOWN;
            }
            else if (target != null && State == BoDState.TeleportLocate && DateTime.Now > teleport_time) {
                teleport_time = DateTime.Now + TELEPORT_LOCATE;
                State = BoDState.Teleporting;
                teleportLocation = target.transform.position;
            } 
            else if (State == BoDState.Teleporting && DateTime.Now > teleport_time) {
                teleport_time = DateTime.Now + TELEPORT_FINISH;
                State = BoDState.TeleportFinish;
                TeleportAttack(teleportLocation);
            } 
            else if (State == BoDState.TeleportFinish && DateTime.Now > teleport_time) {
                m_Animator.SetTrigger("walk");
                State = BoDState.Walk;
                
            } 
            else if (target != null && (State == BoDState.Walk || State == BoDState.Idle)){
                var movement = (new Vector2(target.position.x, target.position.y) - rb.position).normalized;
                TryMove(movement);
                RotateAnimation(-movement);
            }
        }
    }

    public partial class BringerOfDeathController : EnemyController {

            protected void CastSpell(Vector3 spellLocation) {
                var spell = PrefabService.SINGLETON.Instance(PROJECTILE_NAME, spellLocation);
                spellCollider = spell.transform.GetChild(0).GetComponent<EmptyCollider>();
                spellCollider.OnColliderEnter2D_Action += Spell_onColliderEnter2D;
            }
            
            protected void Spell_onColliderEnter2D(Collider2D other) {
                var player = other.gameObject.GetComponent<PlayerController>();
			    if(player != null) {
                    // Create attack object.
                    var attack = (AttackStats) new AttackStats{
                        Damage = 1,
                        Knockback = 0,
                        Range = 0,
                        AttackSpeed = 0,
                        Source = transform.position,
                        Actor = this,
                    }.Calculate(Stats);

                    // Get the player to take the damage.
                    if(player.TakeDamage(attack)){
                        //rb position - player position
                        //Vector2 difference = (other.transform.position - transform.position).normalized;
                        //player.KnockBack(KnockbackForce, difference);
                        //animer nå ?
                    }
                }
            }

            protected void TeleportAttack(Vector3 teleportLocation) {
                if (transform.position.x > teleportLocation.x) {
                    transform.position = teleportLocation + new Vector3(2, 0, 0);
                } else {
                    transform.position = teleportLocation + new Vector3(-2, 0, 0);
                }
                m_Animator.SetTrigger("attack");
            }
        }

    public enum BoDState {
        Idle, Walk, Cast, Casting, CastLocate, CastFinish, Teleport, Teleporting, TeleportLocate, TeleportFinish, Attack
    }
}